@@ -93,13 +93,16 @@ class ConductorAPI(object):
     |    1.42 - Added optional agent_version to heartbeat
     |    1.43 - Added do_node_rescue, do_node_unrescue and can_send_rescue
     |    1.44 - Added add_node_traits and remove_node_traits.
+    |    1.45 - Added continue_node_deploy
+    |    1.46 - Added reset_interfaces to update_node
+    |    1.47 - Added support for conductor groups
 
     """
 
     # NOTE(rloo): This must be in sync with manager.ConductorManager's.
     # NOTE(pas-ha): This also must be in sync with
     #               ironic.common.release_mappings.RELEASE_MAPPING['master']
-    RPC_API_VERSION = '1.44'
+    RPC_API_VERSION = '1.47'
 
     def __init__(self, topic=None):
         super(ConductorAPI, self).__init__()
@@ -115,8 +118,10 @@ class ConductorAPI(object):
                        else self.RPC_API_VERSION)
         self.client = rpc.get_client(target, version_cap=version_cap,
                                      serializer=serializer)
+
+        use_groups = self.client.can_send_version('1.47')
         # NOTE(deva): this is going to be buggy
-        self.ring_manager = hash_ring.HashRingManager()
+        self.ring_manager = hash_ring.HashRingManager(use_groups=use_groups)
 
     def get_topic_for(self, node):
         """Get the RPC topic for the conductor service the node is mapped to.
@@ -127,13 +132,15 @@ class ConductorAPI(object):
 
         """
         try:
-            ring = self.ring_manager[node.driver]
+            ring = self.ring_manager.get_ring(node.driver,
+                                              node.conductor_group)
             dest = ring.get_nodes(node.uuid.encode('utf-8'),
                                   replicas=CONF.hash_distribution_replicas)
             return '%s.%s' % (self.topic, dest.pop())
         except exception.DriverNotFound:
             reason = (_('No conductor service registered which supports '
-                        'driver %s.') % node.driver)
+                        'driver %(driver)s for conductor group "%(group)s".') %
+                      {'driver': node.driver, 'group': node.conductor_group})
             raise exception.NoValidHost(reason=reason)
 
     def get_topic_for_driver(self, driver_name):
@@ -148,13 +155,17 @@ class ConductorAPI(object):
         :raises: DriverNotFound
 
         """
+        # NOTE(jroll) we want to be able to route this to any conductor,
+        # regardless of groupings. We use a fresh, uncached hash ring that
+        # does not take groups into account.
+        local_ring_manager = hash_ring.HashRingManager(use_groups=False,
+                                                       cache=False)
         try:
-            ring = self.ring_manager[driver_name]
+            ring = local_ring_manager.get_ring(driver_name, '')
         except exception.TemporaryFailure:
             # NOTE(dtantsur): even if no conductors are registered, it makes
             # sense to report 404 on any driver request.
             raise exception.DriverNotFound(_("No conductors registered."))
-
         host = random.choice(list(ring.nodes))
         return self.topic + "." + host
 
@@ -183,7 +194,8 @@ class ConductorAPI(object):
         cctxt = self.client.prepare(topic=topic or self.topic, version='1.36')
         return cctxt.call(context, 'create_node', node_obj=node_obj)
 
-    def update_node(self, context, node_obj, topic=None):
+    def update_node(self, context, node_obj, topic=None,
+                    reset_interfaces=False):
         """Synchronously, have a conductor update the node's information.
 
         Update the node's information in the database and return a node object.
@@ -198,13 +210,16 @@ class ConductorAPI(object):
         :param context: request context.
         :param node_obj: a changed (but not saved) node object.
         :param topic: RPC topic. Defaults to self.topic.
+        :param reset_interfaces: whether to reset hardware interfaces to their
+                                 defaults.
         :returns: updated node object, including all fields.
         :raises: NoValidDefaultForInterface if no default can be calculated
                  for some interfaces, and explicit values must be provided.
 
         """
         cctxt = self.client.prepare(topic=topic or self.topic, version='1.1')
-        return cctxt.call(context, 'update_node', node_obj=node_obj)
+        return cctxt.call(context, 'update_node', node_obj=node_obj,
+                          reset_interfaces=reset_interfaces)
 
     def change_node_power_state(self, context, node_id, new_state,
                                 topic=None, timeout=None):
@@ -422,6 +437,20 @@ class ConductorAPI(object):
         return cctxt.cast(context, 'continue_node_clean',
                           node_id=node_id)
 
+    def continue_node_deploy(self, context, node_id, topic=None):
+        """Signal to conductor service to start the next deployment action.
+
+        NOTE(rloo): this is an RPC cast, there will be no response or
+        exception raised by the conductor for this RPC.
+
+        :param context: request context.
+        :param node_id: node id or uuid.
+        :param topic: RPC topic. Defaults to self.topic.
+        """
+        cctxt = self.client.prepare(topic=topic or self.topic, version='1.45')
+        return cctxt.cast(context, 'continue_node_deploy',
+                          node_id=node_id)
+
     def validate_driver_interfaces(self, context, node_id, topic=None):
         """Validate the `core` and `standardized` interfaces for drivers.
 
