@@ -181,6 +181,10 @@ def get_deploy_info(node, address, iqn, port=None, lun='1'):
                        'boot_option': deploy_utils.get_boot_option(node),
                        'boot_mode': _get_boot_mode(node)})
 
+        cpu_arch = node.properties.get('cpu_arch')
+        if cpu_arch is not None:
+            params['cpu_arch'] = cpu_arch
+
         # Append disk label if specified
         disk_label = deploy_utils.get_disk_label(node)
         if disk_label is not None:
@@ -408,7 +412,10 @@ class AgentDeployMixin(agent_base_vendor.AgentDeployMixin):
         uuid_dict_returned = do_agent_iscsi_deploy(task, self._client)
         root_uuid = uuid_dict_returned.get('root uuid')
         efi_sys_uuid = uuid_dict_returned.get('efi system partition uuid')
-        self.prepare_instance_to_boot(task, root_uuid, efi_sys_uuid)
+        prep_boot_part_uuid = uuid_dict_returned.get(
+            'PrEP Boot partition uuid')
+        self.prepare_instance_to_boot(task, root_uuid, efi_sys_uuid,
+                                      prep_boot_part_uuid=prep_boot_part_uuid)
         self.reboot_and_finish_deploy(task)
 
 
@@ -445,6 +452,7 @@ class ISCSIDeploy(AgentDeployMixin, base.DeployInterface):
         validate(task)
 
     @METRICS.timer('ISCSIDeploy.deploy')
+    @base.deploy_step(priority=100)
     @task_manager.require_exclusive_lock
     def deploy(self, task):
         """Start deployment of the task's node.
@@ -474,9 +482,8 @@ class ISCSIDeploy(AgentDeployMixin, base.DeployInterface):
             task.driver.network.configure_tenant_networks(task)
             task.driver.boot.prepare_instance(task)
             manager_utils.node_power_action(task, states.POWER_ON)
-            task.process_event('done')
-            LOG.info('Deployment to node %s done', node.uuid)
-            return states.DEPLOYDONE
+
+            return None
 
     @METRICS.timer('ISCSIDeploy.tear_down')
     @task_manager.require_exclusive_lock
