@@ -26,7 +26,6 @@ from ironic.common import exception
 from ironic.common import keystone
 from ironic.conductor import task_manager
 from ironic.tests import base
-from ironic.tests.unit.conductor import mgr_utils
 from ironic.tests.unit.db import base as db_base
 from ironic.tests.unit.objects import utils as object_utils
 
@@ -101,8 +100,6 @@ class TestCinderClient(base.TestCase):
     def test_get_client_deprecated_opts(self, mock_client_init, mock_session,
                                         mock_auth, mock_sauth, mock_adapter):
 
-        self.config(region_name='test-region',
-                    group='keystone')
         self.config(url='http://test-url', group='cinder')
         mock_adapter.return_value = mock_adapter_obj = mock.Mock()
         mock_adapter_obj.get_endpoint.return_value = 'http://test-url'
@@ -112,7 +109,7 @@ class TestCinderClient(base.TestCase):
         mock_session.assert_called_once_with('cinder')
         mock_adapter.assert_called_once_with(
             'cinder', session=mock.sentinel.session, auth=mock.sentinel.auth,
-            endpoint_override='http://test-url', region_name='test-region')
+            endpoint_override='http://test-url')
         self.assertFalse(mock_sauth.called)
 
 
@@ -120,8 +117,6 @@ class TestCinderUtils(db_base.DbTestCase):
 
     def setUp(self):
         super(TestCinderUtils, self).setUp()
-        mgr_utils.mock_the_extension_manager(driver='fake')
-        self.config(enabled_drivers=['fake'])
         self.node = object_utils.create_test_node(
             self.context,
             instance_uuid=uuidutils.generate_uuid())
@@ -197,8 +192,6 @@ class TestCinderActions(db_base.DbTestCase):
 
     def setUp(self):
         super(TestCinderActions, self).setUp()
-        mgr_utils.mock_the_extension_manager(driver='fake')
-        self.config(enabled_drivers=['fake'])
         self.node = object_utils.create_test_node(
             self.context,
             instance_uuid=uuidutils.generate_uuid())
