@@ -418,6 +418,15 @@ class IscsiDeployMethodsTestCase(db_base.DbTestCase):
         ret_val = self._test_get_deploy_info(extra_instance_info=capabilities)
         self.assertEqual('local', ret_val['boot_option'])
 
+    def test_get_deploy_info_cpu_arch(self):
+        ret_val = self._test_get_deploy_info()
+        self.assertEqual('x86_64', ret_val['cpu_arch'])
+
+    def test_get_deploy_info_cpu_arch_none(self):
+        self.node.properties['cpu_arch'] = None
+        ret_val = self._test_get_deploy_info()
+        self.assertNotIn('cpu_arch', ret_val)
+
     def test_get_deploy_info_disk_label(self):
         capabilities = {'capabilities': {'disk_label': 'msdos'}}
         ret_val = self._test_get_deploy_info(extra_instance_info=capabilities)
@@ -739,17 +748,20 @@ class ISCSIDeployTestCase(db_base.DbTestCase):
                                                     mock_write):
         mock_write.return_value = False
         self.node.provision_state = states.DEPLOYING
+        self.node.deploy_step = {
+            'step': 'deploy', 'priority': 50, 'interface': 'deploy'}
         self.node.save()
         with task_manager.acquire(self.context,
                                   self.node.uuid, shared=False) as task:
-            state = task.driver.deploy.deploy(task)
-            self.assertEqual(state, states.DEPLOYDONE)
+            ret = task.driver.deploy.deploy(task)
+            self.assertIsNone(ret)
             self.assertFalse(mock_cache_instance_image.called)
             self.assertFalse(mock_check_image_size.called)
             mock_remove_network.assert_called_once_with(mock.ANY, task)
             mock_tenant_network.assert_called_once_with(mock.ANY, task)
             mock_prepare_instance.assert_called_once_with(mock.ANY, task)
             self.assertEqual(2, mock_node_power_action.call_count)
+            self.assertEqual(states.DEPLOYING, task.node.provision_state)
 
     @mock.patch.object(noop_storage.NoopStorage, 'detach_volumes',
                        autospec=True)
@@ -893,7 +905,7 @@ class ISCSIDeployTestCase(db_base.DbTestCase):
                 task, task.driver.deploy._client)
             configure_local_boot_mock.assert_called_once_with(
                 task.driver.deploy, task, root_uuid='some-root-uuid',
-                efi_system_part_uuid=None)
+                efi_system_part_uuid=None, prep_boot_part_uuid=None)
             reboot_and_finish_deploy_mock.assert_called_once_with(
                 task.driver.deploy, task)
             set_boot_device_mock.assert_called_once_with(
@@ -925,7 +937,7 @@ class ISCSIDeployTestCase(db_base.DbTestCase):
                 task, task.driver.deploy._client)
             configure_local_boot_mock.assert_called_once_with(
                 task.driver.deploy, task, root_uuid='some-root-uuid',
-                efi_system_part_uuid='efi-part-uuid')
+                efi_system_part_uuid='efi-part-uuid', prep_boot_part_uuid=None)
             reboot_and_finish_deploy_mock.assert_called_once_with(
                 task.driver.deploy, task)
             set_boot_device_mock.assert_called_once_with(
